#!/bin/sh

# Parameters:
# $1 ... JAVA_CMD
# $2 ... JAVA_OPTS
# $3 ... PID_FILE
# $4 ... APP_DATA
# $5 ... SYNOPKG_PKGDEST
# $6 ... MINIM_HOME
# $7 ... MINIM_OPTS
# $8 ... JAVA_BIN
JAVA_CMD="$1"
JAVA_OPTS="$2"
PID_FILE="$3"
APP_DATA="$4"
SYNOPKG_PKGDEST="$5"
MINIM_HOME="$6"
MINIM_OPTS="$7"
JAVA_BIN="$8"

# initialization
MINIM_OPTS="--noprompt --nohup $MINIM_OPTS"
PID_FILE_DIR=$(dirname "$PID_FILE")
[ ! -d "$MINIM_HOME/$APP_DATA" ] && mkdir "$MINIM_HOME/$APP_DATA"
[ ! -e "$PID_FILE_DIR" ] && mkdir "$PID_FILE_DIR"
cd "$MINIM_HOME/$APP_DATA"
OUT_FILE="$MINIM_HOME/$APP_DATA/minimserver-out.log"
rm -f "$OUT_FILE"

# run MinimServer and write the pid of the Java process to PID_FILE
(
    # relaunch protocol
    RELAUNCH=""
    while :
    do
        if [ -n "$JAVA_BIN" ]; then
            JAVAX="$JAVA_BIN"
        else
            JAVAX="$JAVA_CMD"
            for f in "$SYNOPKG_PKGDEST"/lib/minimjre*.dir "$SYNOPKG_PKGDEST"/libext/minimjre*.dir
            do
                [ ! -e "$f.jmdel" ] && [ -x "$f/jre/bin/java" ] && JAVAX="$f/jre/bin/java"
            done
        fi
        if [ -z "$RELAUNCH" ]; then
            LD_LIBRARY_PATH="$SYNOPKG_PKGDEST/libsys" MINIM_LANG="$SYNOPKG_DSM_LANGUAGE" "$JAVAX" ${JAVA_OPTS} -jar "$SYNOPKG_PKGDEST/lib/mserver.jar" --launchrc $MINIM_OPTS --noconsole </dev/null >"$OUT_FILE" 2>&1 &
        else
            LD_LIBRARY_PATH="$SYNOPKG_PKGDEST/libsys" MINIM_LANG="$SYNOPKG_DSM_LANGUAGE" "$JAVAX" ${JAVA_OPTS} -jar "$SYNOPKG_PKGDEST/lib/mserver.jar" --launchrc $RELAUNCH $MINIM_OPTS --noconsole </dev/null >>"$OUT_FILE" 2>&1 &
        fi
        mpid=$!
        echo $mpid > "$PID_FILE"
        wait $mpid
        LAUNCH_RC=$?
        [ $LAUNCH_RC -ne 23 ] && exit $LAUNCH_RC
        RELAUNCH="--relaunch"
    done
) &

# check for a successful MinimServer start
OUT_FIRST_LINE=
COUNTER=15
while [ $COUNTER -ne 0 ]
do
    [ -f "$OUT_FILE" ] && OUT_FIRST_LINE=$(head -n 1 "$OUT_FILE")
    [ -n "$OUT_FIRST_LINE" ] && break
    let COUNTER=COUNTER-1
    sleep 1
done
if [ -n "$OUT_FIRST_LINE" ] && [ "$OUT_FIRST_LINE" != "~-~" ]; then
    synologset1 sys err 0x11800000 "MinimServer start error: $OUT_FIRST_LINE"
    exit 1
fi

exit 0
