#!/bin/sh
##########################################
#                                        #
# MinimServer Synology package           #
#                                        #
##########################################

APP_DATA="appData"
LOG_FILE="/var/run/minimserver-log"
SERVICETOOL="/usr/syno/bin/servicetool"
PACKAGE_LOCATION="/var/packages/${SYNOPKG_PKGNAME}/target"
appstore_target=$(readlink "${PACKAGE_LOCATION}")
if [ ! -z "${appstore_target}" ]; then
	appstore_dir=$(dirname "${appstore_target}")
	VOLUME=$(dirname "${appstore_dir}")
else
	VOLUME=$(/usr/syno/bin/servicetool --get-alive-volume)
fi
MINIM_HOME="${VOLUME}/${SYNOPKG_PKGNAME}"

debug ()
{
	DATE=$(date +"%a %d %b %T")
	echo "${DATE} $1" >>$LOG_FILE
}

##################################################################################

preinst ()
{
	# abort installation if requested by user
	if [ "$install_key" == "false" ]; then
		exit 1
	fi

	exit 0
}

##################################################################################

postinst ()
{
	# create home directory if it doesn't exist
	if [ ! -d "${MINIM_HOME}" ]; then
		synoshare --add "${SYNOPKG_PKGNAME}" "${SYNOPKG_PKGNAME} configuration" "${MINIM_HOME}" "" "@administrators" "@users" 0 0
		synoacltool -add "${MINIM_HOME}" everyone:*:allow:r-x---a-R-c--:-d-- >/dev/null
		synoacltool -add "${MINIM_HOME}" everyone:*:allow:r-----a-R-c--:f--- >/dev/null
	fi
	if [ ! -d "$MINIM_HOME/$APP_DATA" ]; then
		mkdir "$MINIM_HOME/$APP_DATA"
	fi
	if [ ! -d "$MINIM_HOME/etc" ]; then
		mkdir "$MINIM_HOME/etc"
	fi
	if [ ! -d "$MINIM_HOME/opt" ]; then
		mkdir "$MINIM_HOME/opt"
	fi
	if [ ! -d "$MINIM_HOME/opt/bin" ]; then
		mkdir "$MINIM_HOME/opt/bin"
	fi

	# restore the backup config file if it exists
	if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -f "$MINIM_HOME/backup/minimserver.config" ]; then
		backup_config_dir="$MINIM_HOME/backup"
	else
		# need to check the public share if updating from a version of MinimServer prior to 0.8.4.1
		if [ -f "/etc/samba/smb.conf" ]; then
			# location for DSM 6.0 and later
			smb_conf="/etc/samba/smb.conf"
		elif [ -f "/usr/syno/etc/smb.conf" ]; then
			# location for DSM 5.2 and earlier
			smb_conf="/usr/syno/etc/smb.conf"
		fi
		if [ -n "$smb_conf" ]; then
			public_dir=$(cat "$smb_conf" | sed -r '/\/'public'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'public').*$/\1/')
			if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -n "$public_dir" -a -f "${public_dir}/minimserver/minimserver.config" ]; then
				backup_config_dir="${public_dir}/minimserver"
			fi
		fi
	fi
	if [ -n "${backup_config_dir}" ]; then
		# there should not be a config file in appData, but check for this to be safe
		if [ -f "$MINIM_HOME/$APP_DATA/minimserver.config" ]; then
			rm "$MINIM_HOME/$APP_DATA/minimserver.config"
		fi
		# use the backup config file
		mv "${backup_config_dir}/minimserver.config" "$MINIM_HOME/$APP_DATA"
		# remove the configuration backup directory
		rm -r "${backup_config_dir}"
	fi

	# install the service configure file
	#${SERVICETOOL} --install-configure-file --package ${PACKAGE_LOCATION}/etc/MinimServer.sc >/dev/null

	exit 0
}

##################################################################################

preuninst ()
{
	debug "called preuninst: sps=${SYNOPKG_PKG_STATUS}"
	if [ "${SYNOPKG_PKG_STATUS}" == "UNINSTALL" ]; then
		grep -q ".licenseType = full" ${MINIM_HOME}/${APP_DATA}/minimserver.config
		is_full_license=$?
		debug "preuninst: is_full_license=$is_full_license"
		if [ $is_full_license -eq 0 ]; then
			grep -q "minimserver.deactivateLicenseError = true" ${MINIM_HOME}/${APP_DATA}/minimserver.config
			is_deactivate_error=$?
			debug "preuninst: is_deactivate_error=$is_deactivate_error"
			if [ $is_deactivate_error -ne 0 ]; then
				echo "You must deactivate your MinimServer full license before uninstalling." >${SYNOPKG_TEMP_LOGFILE}
				exit 1
			fi
		fi
	fi

	# if upgrading, make a backup of MinimServer application data and other directories
	if [ "${SYNOPKG_PKG_STATUS}" == "UPGRADE" ]; then
		if [ ! -d "${VOLUME}/@tmp/minimtmp" ]; then
			mkdir ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${MINIM_HOME}/${APP_DATA}" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/${APP_DATA}" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/${APP_DATA}"
			fi
			cp -pr ${MINIM_HOME}/${APP_DATA} ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${MINIM_HOME}/opt" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/opt" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/opt"
			fi
			cp -pr ${MINIM_HOME}/opt ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${PACKAGE_LOCATION}/libext" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/libext" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/libext"
			fi
			cp -pr ${PACKAGE_LOCATION}/libext ${VOLUME}/@tmp/minimtmp
		fi
		 if [ -d "${MINIM_HOME}/etc" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/etc" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/etc"
			fi
			cp -pr ${MINIM_HOME}/etc ${VOLUME}/@tmp/minimtmp
fi
	fi

	# backup minimserver.config if chosen in the uninstallation dialog
	backup_dir="${MINIM_HOME}/backup"
	if [ "$backup_key" == "true" ]; then
		# copy the configuration file to the backup directory
		[ ! -d "${backup_dir}" ] && mkdir "${backup_dir}"
		cp -p "${MINIM_HOME}/${APP_DATA}/minimserver.config" "${backup_dir}"
	elif [ "$backup_key" == "false" ]; then
		# remove the configuration backup directory if it exists
		if [ -d "${backup_dir}" ]; then
			rm -r "${backup_dir}"
		fi
	fi

	# remove appData, opt and etc directories
	if [ -d "${MINIM_HOME}/${APP_DATA}" ]; then
		rm -r "${MINIM_HOME}/${APP_DATA}"
	fi
	if [ -d "${MINIM_HOME}/opt" ]; then
		rm -r "${MINIM_HOME}/opt"
	fi
	if [ -d "${MINIM_HOME}/etc" ]; then
		ETC_FILES=$(ls "${MINIM_HOME}/etc"/ServerBundle_*.defaults 2>/dev/null)
		[ -n "${ETC_FILES}" ] && rm -f "${MINIM_HOME}"/etc/ServerBundle_*.defaults
		# don't remove etc directory if it contains user-created files
		[ -z "$(ls -A "${MINIM_HOME}/etc")" ] && rmdir "${MINIM_HOME}/etc"
	fi

	# remove home directory if uninstalling
	if [ "${SYNOPKG_PKG_STATUS}" == "UNINSTALL" ]; then
		# don't remove home directory if it contains a configuration backup or user-created files
		if [ "$(ls -A "${MINIM_HOME}")" = @eaDir ]; then
			rm -rf "${MINIM_HOME}/@eaDir"
		fi
		[ -z "$(ls -A "${MINIM_HOME}")" ] && synoshare --del TRUE "${SYNOPKG_PKGNAME}"
	fi

	# uninstall the service configure file
	#${SERVICETOOL} --remove-configure-file --package MinimServer.sc >/dev/null

	exit 0
}

##################################################################################

postuninst ()
{
	exit 0
}

##################################################################################

preupgrade ()
{
	exit 0
}

##################################################################################

postupgrade ()
{
	# restore the backup of MinimServer application data and other directories
	if [ -d "${VOLUME}/@tmp/minimtmp" ]; then
		if [ -d "${VOLUME}/@tmp/minimtmp/${APP_DATA}" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/${APP_DATA} ${MINIM_HOME}
			# the files in appData might have been created by the daemon user
			chown -R "root:root" ${MINIM_HOME}/${APP_DATA}
		fi
		if [ -d "${VOLUME}/@tmp/minimtmp/opt" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/opt ${MINIM_HOME}
		fi
		if [ -d "${VOLUME}/@tmp/minimtmp/libext" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/libext ${PACKAGE_LOCATION}
			# the files in libext might have been created by the daemon user
			chown -R "root:root" ${VOLUME}/@tmp/minimtmp/libext
		fi
		if [ -d "${VOLUME}/@tmp/minimtmp/etc" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/etc ${MINIM_HOME}
		fi
		rm -r ${VOLUME}/@tmp/minimtmp
	fi

	exit 0
}
